(*
 * Copyright (c) 2003 Fabrice Bellard
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 *)

(**
 * @file
 * libavformat API example.
 *
 * Output a media file in any supported libavformat format. The default
 * codecs are used.
 * @example muxing.c
 *)

(*
 * FFVCL - Delphi FFmpeg VCL Components
 * http://www.DelphiFFmpeg.com
 *
 * Original file: doc/examples/muxing.c
 * Ported by CodeCoolie@CNSW 2014/08/30 -> $Date:: 2014-09-02 #$
 *)

(*
FFmpeg Delphi/Pascal Headers and Examples License Agreement

A modified part of FFVCL - Delphi FFmpeg VCL Components.
Copyright (c) 2008-2014 DelphiFFmpeg.com
All rights reserved.
http://www.DelphiFFmpeg.com

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:
1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.

This source code is provided "as is" by DelphiFFmpeg.com without
warranty of any kind, either expressed or implied, including but not
limited to the implied warranties of merchantability and/or fitness
for a particular purpose.

Please also notice the License agreement of FFmpeg libraries.
*)

program muxing;

{$APPTYPE CONSOLE}

{$IFDEF FPC}
  {$MODE Delphi}
{$ENDIF}

uses
{$IFDEF FPC}
  {$IFDEF MSWINDOWS}
    Windows,
  {$ENDIF}
  SysUtils,
{$ELSE}
  {$IF CompilerVersion >= 23.0}
    {$IFDEF MSWINDOWS}
      Winapi.Windows,
    {$ENDIF}
    System.SysUtils,
  {$ELSE}
    {$IFDEF MSWINDOWS}
      Windows,
    {$ENDIF}
    SysUtils,
  {$IFEND}
{$ENDIF}

  FFTypes,
  FFUtils,

  libavcodec,
  libavcodec_avfft,
  libavdevice,
  libavfilter,
  libavfilter_avcodec,
  libavfilter_buffersink,
  libavfilter_buffersrc,
  libavfilter_formats,
  libavformat,
  libavformat_avio,
  libavformat_url,
  libavutil,
  libavutil_audio_fifo,
  libavutil_avstring,
  libavutil_bprint,
  libavutil_buffer,
  libavutil_channel_layout,
  libavutil_common,
  libavutil_cpu,
  libavutil_dict,
  libavutil_error,
  libavutil_eval,
  libavutil_fifo,
  libavutil_file,
  libavutil_frame,
  libavutil_imgutils,
  libavutil_log,
  libavutil_mathematics,
  libavutil_md5,
  libavutil_mem,
  libavutil_opt,
  libavutil_parseutils,
  libavutil_pixdesc,
  libavutil_pixfmt,
  libavutil_rational,
  libavutil_samplefmt,
  libavutil_time,
  libavutil_timestamp,
  libswresample,
  libswscale;

var
  audio_is_eof, video_is_eof: Integer;

const
  STREAM_DURATION   = 10.0;
  STREAM_FRAME_RATE = 25; (* 25 images/s *)
  STREAM_PIX_FMT    = AV_PIX_FMT_YUV420P; (* default pix_fmt *)

var
  sws_flags: Integer = SWS_BICUBIC;

procedure log_packet(const fmt_ctx: PAVFormatContext; const pkt: PAVPacket);
var
  time_base: PAVRational;
begin
  time_base := @PAVStream(PPtrIdx(fmt_ctx.streams, pkt.stream_index)).time_base;

  Writeln(Format('pts:%s pts_time:%s dts:%s dts_time:%s duration:%s duration_time:%s stream_index:%d',
        [av_ts2str(pkt.pts), av_ts2timestr(pkt.pts, time_base),
         av_ts2str(pkt.dts), av_ts2timestr(pkt.dts, time_base),
         av_ts2str(pkt.duration), av_ts2timestr(pkt.duration, time_base),
         pkt.stream_index]));
end;

function write_frame(fmt_ctx: PAVFormatContext; const time_base: PAVRational; st: PAVStream; pkt: PAVPacket): Integer;
begin
  (* rescale output packet timestamp values from codec to stream timebase *)
  pkt.pts := av_rescale_q_rnd(pkt.pts, time_base^, st.time_base, TAVRounding(Ord(AV_ROUND_NEAR_INF) or Ord(AV_ROUND_PASS_MINMAX)));
  pkt.dts := av_rescale_q_rnd(pkt.dts, time_base^, st.time_base, TAVRounding(Ord(AV_ROUND_NEAR_INF) or Ord(AV_ROUND_PASS_MINMAX)));
  pkt.duration := av_rescale_q(pkt.duration, time_base^, st.time_base);
  pkt.stream_index := st.index;

  (* Write the compressed frame to the media file. *)
  log_packet(fmt_ctx, pkt);
  Result := av_interleaved_write_frame(fmt_ctx, pkt);
end;

(* Add an output stream. *)
function add_stream(oc: PAVFormatContext; codec: PPAVCodec;
  codec_id: TAVCodecID): PAVStream;
var
  c: PAVCodecContext;
  st: PAVStream;
begin
  (* find the encoder *)
  codec^ := avcodec_find_encoder(codec_id);
  if not Assigned(codec^) then
  begin
    Writeln(ErrOutput, Format('Could not find encoder for ''%s''',
           [avcodec_get_name(codec_id)]));
    Result := nil;
    Exit;
  end;

  st := avformat_new_stream(oc, codec^);
  if not Assigned(st) then
  begin
    Writeln(ErrOutput, 'Could not allocate stream');
    Result := nil;
    Exit;
  end;
  st.id := oc.nb_streams-1;
  c := st.codec;

  case codec^.ttype of
    AVMEDIA_TYPE_AUDIO:
    begin
      if Assigned(codec^.sample_fmts) then
        c.sample_fmt := codec^.sample_fmts^
      else
        c.sample_fmt := AV_SAMPLE_FMT_FLTP;
      c.bit_rate    := 64000;
      c.sample_rate := 44100;
      c.channels    := 2;
    end;
    AVMEDIA_TYPE_VIDEO:
    begin
      c.codec_id := codec_id;

      c.bit_rate := 400000;
      (* Resolution must be a multiple of two. *)
      c.width    := 352;
      c.height   := 288;
      (* timebase: This is the fundamental unit of time (in seconds) in terms
       * of which frame timestamps are represented. For fixed-fps content,
       * timebase should be 1/framerate and timestamp increments should be
       * identical to 1. *)
      c.time_base.den := STREAM_FRAME_RATE;
      c.time_base.num := 1;
      c.gop_size      := 12; (* emit one intra frame every twelve frames at most *)
      c.pix_fmt       := STREAM_PIX_FMT;
      if c.codec_id = AV_CODEC_ID_MPEG2VIDEO then
        (* just for testing, we also add B frames *)
        c.max_b_frames := 2;
      if c.codec_id = AV_CODEC_ID_MPEG1VIDEO then
        (* Needed to avoid using macroblocks in which some coeffs overflow.
         * This does not happen with normal video, it just happens here as
         * the motion of the chroma plane does not match the luma plane. *)
        c.mb_decision := 2;
    end;
  end;

  (* Some formats want stream headers to be separate. *)
  if (oc.oformat.flags and AVFMT_GLOBALHEADER) <> 0 then
    c.flags := c.flags or CODEC_FLAG_GLOBAL_HEADER;

  Result := st;
end;

(**************************************************************)
(* audio output *)

var
  t, tincr, tincr2: Single;

  audio_frame: PAVFrame;
  src_samples_data: PPByte;
  src_samples_linesize: Integer;
  src_nb_samples: Integer;

  max_dst_nb_samples: Integer;
  dst_samples_data: PPByte;
  dst_samples_linesize: Integer;
  dst_samples_size: Integer;
  samples_count: Integer;

  swr_ctx: PSwrContext = nil;

function open_audio(oc: PAVFormatContext; codec: PAVCodec; st: PAVStream): Integer;
var
  c: PAVCodecContext;
  ret: Integer;
begin
  c := st.codec;

  (* allocate and init a re-usable frame *)
  audio_frame := av_frame_alloc();
  if not Assigned(audio_frame) then
  begin
    Writeln(ErrOutput, 'Could not allocate audio frame');
    Result := -1;
    Exit;
  end;

  (* open it *)
  ret := avcodec_open2(c, codec, nil);
  if ret < 0 then
  begin
    Writeln(ErrOutput, Format('Could not open audio codec: %s', [av_err2str(ret)]));
    Result := -1;
    Exit;
  end;

  (* init signal generator *)
  t     := 0;
  tincr := 2 * M_PI * 110.0 / c.sample_rate;
  (* increment frequency by 110 Hz per second *)
  tincr2 := 2 * M_PI * 110.0 / c.sample_rate / c.sample_rate;

  if (c.codec.capabilities and CODEC_CAP_VARIABLE_FRAME_SIZE) <> 0 then
    src_nb_samples := 10000
  else
    src_nb_samples := c.frame_size;

  ret := av_samples_alloc_array_and_samples(@src_samples_data, @src_samples_linesize, c.channels,
                                           src_nb_samples, AV_SAMPLE_FMT_S16, 0);
  if ret < 0 then
  begin
    Writeln(ErrOutput, 'Could not allocate source samples');
    Result := -1;
    Exit;
  end;

  (* compute the number of converted samples: buffering is avoided
   * ensuring that the output buffer will contain at least all the
   * converted input samples *)
  max_dst_nb_samples := src_nb_samples;

  (* create resampler context *)
  if c.sample_fmt <> AV_SAMPLE_FMT_S16 then
  begin
    swr_ctx := swr_alloc();
    if not Assigned(swr_ctx) then
    begin
      Writeln(ErrOutput, 'Could not allocate resampler context');
      Result := -1;
      Exit;
    end;

    (* set options *)
    av_opt_set_int       (swr_ctx, 'in_channel_count',   c.channels,       0);
    av_opt_set_int       (swr_ctx, 'in_sample_rate',     c.sample_rate,    0);
    av_opt_set_sample_fmt(swr_ctx, 'in_sample_fmt',      AV_SAMPLE_FMT_S16, 0);
    av_opt_set_int       (swr_ctx, 'out_channel_count',  c.channels,       0);
    av_opt_set_int       (swr_ctx, 'out_sample_rate',    c.sample_rate,    0);
    av_opt_set_sample_fmt(swr_ctx, 'out_sample_fmt',     c.sample_fmt,     0);

    (* initialize the resampling context *)
    ret := swr_init(swr_ctx);
    if ret < 0 then
    begin
      Writeln(ErrOutput, 'Failed to initialize the resampling context');
      Result := -1;
      Exit;
    end;

    ret := av_samples_alloc_array_and_samples(@dst_samples_data, @dst_samples_linesize, c.channels,
                                             max_dst_nb_samples, c.sample_fmt, 0);
    if ret < 0 then
    begin
      Writeln(ErrOutput, 'Could not allocate destination samples');
      Result := -1;
      Exit;
    end;
  end
  else
    dst_samples_data := src_samples_data;
  dst_samples_size := av_samples_get_buffer_size(nil, c.channels, max_dst_nb_samples,
                                                c.sample_fmt, 0);
  Result := 0;
end;

(* Prepare a 16 bit dummy audio frame of 'frame_size' samples and
 * 'nb_channels' channels. *)
procedure get_audio_frame(samples: PSmallInt; frame_size, nb_channels: Integer);
var
  j, i, v: Integer;
  q: PSmallInt;
begin
  q := samples;
  for j := 0 to frame_size - 1 do
  begin
    v := Trunc(sin(t) * 10000);
    for i := 0 to nb_channels - 1 do
    begin
      q^ := v;
      Inc(q);
    end;
    t := t + tincr;
    tincr := tincr + tincr2;
  end;
end;

function write_audio_frame(oc: PAVFormatContext; st: PAVStream; flush: Integer): Integer;
var
  c: PAVCodecContext;
  pkt: TAVPacket;
  got_packet, ret, dst_nb_samples: Integer;
begin
  FillChar(pkt, SizeOf(TAVPacket), 0); // data and size must be 0;

  av_init_packet(@pkt);
  c := st.codec;

  if flush = 0 then
  begin
    get_audio_frame(PSmallInt(src_samples_data^), src_nb_samples, c.channels);

    (* convert samples from native format to destination codec format, using the resampler *)
    if Assigned(swr_ctx) then
    begin
      (* compute destination number of samples *)
      dst_nb_samples := av_rescale_rnd(swr_get_delay(swr_ctx, c.sample_rate) + src_nb_samples,
                                      c.sample_rate, c.sample_rate, AV_ROUND_UP);
      if dst_nb_samples > max_dst_nb_samples then
      begin
        av_free(dst_samples_data^);
        ret := av_samples_alloc(dst_samples_data, @dst_samples_linesize, c.channels,
                               dst_nb_samples, c.sample_fmt, 0);
        if ret < 0 then
        begin
          Result := -1;
          Exit;
        end;
        max_dst_nb_samples := dst_nb_samples;
        dst_samples_size := av_samples_get_buffer_size(nil, c.channels, dst_nb_samples,
                                                      c.sample_fmt, 0);
      end;

      (* convert to destination format *)
      ret := swr_convert(swr_ctx,
                        dst_samples_data, dst_nb_samples,
                        src_samples_data, src_nb_samples);
      if ret < 0 then
      begin
        Writeln(ErrOutput, 'Error while converting');
        Result := -1;
        Exit;
      end;
    end
    else
      dst_nb_samples := src_nb_samples;

    audio_frame.nb_samples := dst_nb_samples;
    audio_frame.pts := av_rescale_q(samples_count, av_make_q(1, c.sample_rate), c.time_base);
    avcodec_fill_audio_frame(audio_frame, c.channels, c.sample_fmt,
                             dst_samples_data^, dst_samples_size, 0);
    Inc(samples_count, dst_nb_samples);

    ret := avcodec_encode_audio2(c, @pkt, audio_frame, @got_packet);
  end
  else
    ret := avcodec_encode_audio2(c, @pkt, nil, @got_packet);

  if ret < 0 then
  begin
    Writeln(ErrOutput, Format('Error encoding audio frame: %s', [av_err2str(ret)]));
    Result := -1;
    Exit;
  end;

  if got_packet = 0 then
  begin
    if flush <> 0 then
      audio_is_eof := 1;
    Result := 0;
    Exit;
  end;

  ret := write_frame(oc, @c.time_base, st, @pkt);
  if ret < 0 then
  begin
    Writeln(ErrOutput, Format('Error while writing audio frame: %s',
            [av_err2str(ret)]));
    Result := -1;
    Exit;
  end;

  Result := 0;
end;

procedure close_audio(oc: PAVFormatContext; st: PAVStream);
begin
  avcodec_close(st.codec);
  if dst_samples_data <> src_samples_data then
  begin
    av_free(dst_samples_data^);
    av_free(dst_samples_data);
  end;
  av_free(src_samples_data^);
  av_free(src_samples_data);
  av_frame_free(@audio_frame);
end;

(**************************************************************)
(* video output *)

var
  frame: PAVFrame;
  src_picture, dst_picture: TAVPicture;
  frame_count: Integer;

function open_video(oc: PAVFormatContext; codec: PAVCodec; st: PAVStream): Integer;
var
  ret: Integer;
  c: PAVCodecContext;
begin
  c := st.codec;

  (* open the codec *)
  ret := avcodec_open2(c, codec, nil);
  if ret < 0 then
  begin
    Writeln(ErrOutput, Format('Could not open video codec: %s', [av_err2str(ret)]));
    Result := -1;
    Exit;
  end;

  (* allocate and init a re-usable frame *)
  frame := av_frame_alloc();
  if not Assigned(frame) then
  begin
    Writeln(ErrOutput, 'Could not allocate video frame');
    Result := -1;
    Exit;
  end;
  frame.format := Ord(c.pix_fmt);
  frame.width := c.width;
  frame.height := c.height;

  (* Allocate the encoded raw picture. *)
  ret := avpicture_alloc(@dst_picture, c.pix_fmt, c.width, c.height);
  if ret < 0 then
  begin
    Writeln(ErrOutput, Format('Could not allocate picture: %s', [av_err2str(ret)]));
    Result := -1;
    Exit;
  end;

  (* If the output format is not YUV420P, then a temporary YUV420P
   * picture is needed too. It is then converted to the required
   * output format. *)
  if c.pix_fmt <> AV_PIX_FMT_YUV420P then
  begin
    ret := avpicture_alloc(@src_picture, AV_PIX_FMT_YUV420P, c.width, c.height);
    if ret < 0 then
    begin
      Writeln(ErrOutput, Format('Could not allocate temporary picture: %s',
              [av_err2str(ret)]));
      Result := -1;
      Exit;
    end;
  end;

  (* copy data and linesize picture pointers to frame *)
  PAVPicture(frame)^ := dst_picture;

  Result := 0;
end;

(* Prepare a dummy image. *)
procedure fill_yuv_image(pict: PAVPicture; frame_index, width, height: Integer);
var
  x, y, i: Integer;
begin
  i := frame_index;

  (* Y *)
  for y := 0 to height - 1 do
    for x := 0 to width - 1 do
      PByte(@PAnsiChar(pict.data[0])[y * pict.linesize[0] + x])^ := x + y + i * 3;

  (* Cb and Cr *)
  for y := 0 to height div 2 - 1 do
    for x := 0 to width div 2 - 1 do
    begin
      PByte(@PAnsiChar(pict.data[1])[y * pict.linesize[1] + x])^ := 128 + y + i * 2;
      PByte(@PAnsiChar(pict.data[2])[y * pict.linesize[2] + x])^ := 64 + x + i * 5;
    end;
end;

var
  sws_ctx: PSwsContext = nil;

function write_video_frame(oc: PAVFormatContext; st: PAVStream; flush: Integer): Integer;
var
  ret: Integer;
  c: PAVCodecContext;
  pkt: TAVPacket;
  got_packet: Integer;
begin
  c := st.codec;

  if flush = 0 then
  begin
    if c.pix_fmt <> AV_PIX_FMT_YUV420P then
    begin
      (* as we only generate a YUV420P picture, we must convert it
       * to the codec pixel format if needed *)
      if not Assigned(sws_ctx) then
      begin
        sws_ctx := sws_getContext(c.width, c.height, AV_PIX_FMT_YUV420P,
                                 c.width, c.height, c.pix_fmt,
                                 sws_flags, nil, nil, nil);
        if not Assigned(sws_ctx) then
        begin
          Writeln(ErrOutput,
                  'Could not initialize the conversion context');
          Result := -1;
          Exit;
        end;
      end;
      fill_yuv_image(@src_picture, frame_count, c.width, c.height);
      sws_scale(sws_ctx,
                @src_picture.data[0], @src_picture.linesize[0],
                0, c.height, @dst_picture.data[0], @dst_picture.linesize[0]);
    end
    else
      fill_yuv_image(@dst_picture, frame_count, c.width, c.height);
  end;

  if ((oc.oformat.flags and AVFMT_RAWPICTURE) <> 0) and (flush = 0) then
  begin
    (* Raw video case - directly store the picture in the packet *)
    av_init_packet(@pkt);

    pkt.flags        := pkt.flags or AV_PKT_FLAG_KEY;
    pkt.stream_index := st.index;
    pkt.data         := dst_picture.data[0];
    pkt.size         := SizeOf(TAVPicture);

    ret := av_interleaved_write_frame(oc, @pkt);
  end
  else
  begin
    FillChar(pkt, SizeOf(TAVPacket), 0);
    av_init_packet(@pkt);

    (* encode the image *)
    frame.pts := frame_count;
    if flush <> 0 then
      ret := avcodec_encode_video2(c, @pkt, nil, @got_packet)
    else
      ret := avcodec_encode_video2(c, @pkt, frame, @got_packet);

    if ret < 0 then
    begin
      Writeln(ErrOutput, Format('Error encoding video frame: %s', [av_err2str(ret)]));
      Result := -1;
      Exit;
    end;
    (* If size is zero, it means the image was buffered. *)

    if got_packet <> 0 then
      ret := write_frame(oc, @c.time_base, st, @pkt)
    else
    begin
      if flush <> 0 then
        video_is_eof := 1;
      ret := 0;
    end;
  end;

  if ret < 0 then
  begin
    Writeln(ErrOutput, Format('Error while writing video frame: %s', [av_err2str(ret)]));
    Result := -1;
    Exit;
  end;
  Inc(frame_count);

  Result := 0;
end;

procedure close_video(oc: PAVFormatContext; st: PAVStream);
begin
  avcodec_close(st.codec);
  av_free(src_picture.data[0]);
  av_free(dst_picture.data[0]);
  av_frame_free(@frame);
end;

(**************************************************************)
(* media file output *)

function main(): Integer;
var
  filename: string;
  fmt: PAVOutputFormat;
  oc: PAVFormatContext;
  audio_st, video_st: PAVStream;
  audio_codec, video_codec: PAVCodec;
  audio_time, video_time: Double;
  flush, ret: Integer;
begin
  Set8087CW($133F); { Disable all fpu exceptions }
  (* Initialize libavcodec, and register all codecs and formats. *)
  av_register_all();

  if ParamCount <> 1 then
  begin
    Writeln(Format('usage: %s output_file' + sLineBreak +
           'API example program to output a media file with libavformat.' + sLineBreak +
           'This program generates a synthetic audio and video stream, encodes and' + sLineBreak +
           'muxes them into a file named output_file.' + sLineBreak +
           'The output format is automatically guessed according to the file extension.' + sLineBreak +
           'Raw images can also be output by using ''%%d'' in the filename.',
           [ExtractFileName(ParamStr(0))]));
    Result := 1;
    Exit;
  end;

  filename := ParamStr(1);

  (* allocate the output media context *)
  avformat_alloc_output_context2(@oc, nil, nil, PAnsiChar(AnsiString(filename)));
  if not Assigned(oc) then
  begin
    Writeln('Could not deduce output format from file extension: using MPEG.');
    avformat_alloc_output_context2(@oc, nil, 'mpeg', PAnsiChar(AnsiString(filename)));
  end;
  if not Assigned(oc) then
  begin
    Result := 1;
    Exit;
  end;

  fmt := oc.oformat;

  (* Add the audio and video streams using the default format codecs
   * and initialize the codecs. *)
  video_st := nil;
  audio_st := nil;

  if fmt.video_codec <> AV_CODEC_ID_NONE then
  begin
    video_st := add_stream(oc, @video_codec, fmt.video_codec);
    if not Assigned(video_st) then
    begin
      Result := 1;
      Exit;
    end;
  end;
  if fmt.audio_codec <> AV_CODEC_ID_NONE then
  begin
    audio_st := add_stream(oc, @audio_codec, fmt.audio_codec);
    if not Assigned(audio_st) then
    begin
      Result := 1;
      Exit;
    end;
  end;

  (* Now that all the parameters are set, we can open the audio and
   * video codecs and allocate the necessary encode buffers. *)
  if Assigned(video_st) then
    if open_video(oc, video_codec, video_st) < 0 then
    begin
      Result := 1;
      Exit;
    end;
  if Assigned(audio_st) then
    if open_audio(oc, audio_codec, audio_st) < 0 then
    begin
      Result := 1;
      Exit;
    end;

  av_dump_format(oc, 0, PAnsiChar(AnsiString(filename)), 1);
  (* open the output file, if needed *)
  if (fmt.flags and AVFMT_NOFILE) = 0 then
  begin
    ret := avio_open(@oc.pb, PAnsiChar(AnsiString(filename)), AVIO_FLAG_WRITE);
    if ret < 0 then
    begin
      Writeln(ErrOutput, Format('Could not open ''%s'': %s', [filename,
              av_err2str(ret)]));
      Result := 1;
      Exit;
    end;
  end;

  (* Write the stream header, if any. *)
  ret := avformat_write_header(oc, nil);
  if ret < 0 then
  begin
    Writeln(ErrOutput, Format('Error occurred when opening output file: %s',
            [av_err2str(ret)]));
    Result := 1;
    Exit;
  end;

  flush := 0;
  while (Assigned(video_st) and (video_is_eof = 0)) or (Assigned(audio_st) and (audio_is_eof = 0)) do
  begin
    (* Compute current audio and video time. *)
    if Assigned(audio_st) and (audio_is_eof = 0) then
      audio_time := audio_st.pts.val * av_q2d(audio_st.time_base)
    else
      audio_time := av_int2float($7fc00000); // INFINITY;
    if Assigned(video_st) and (video_is_eof = 0) then
      video_time := video_st.pts.val * av_q2d(video_st.time_base)
    else
      video_time := av_int2float($7fc00000); // INFINITY;

    if (flush = 0) and
      (not Assigned(audio_st) or (audio_time >= STREAM_DURATION)) and
      (not Assigned(video_st) or (video_time >= STREAM_DURATION)) then
      flush := 1;

    (* write interleaved audio and video frames *)
    if Assigned(audio_st) and (audio_is_eof = 0) and (audio_time <= video_time) then
    begin
      if write_audio_frame(oc, audio_st, flush) < 0 then
      begin
        Result := 1;
        Exit;
      end;
    end
    else if Assigned(video_st) and (video_is_eof = 0) and (video_time < audio_time) then
    begin
      if write_video_frame(oc, video_st, flush) < 0 then
      begin
        Result := 1;
        Exit;
      end;
    end;
  end;

  (* Write the trailer, if any. The trailer must be written before you
   * close the CodecContexts open when you wrote the header; otherwise
   * av_write_trailer() may try to use memory that was freed on
   * av_codec_close(). *)
  av_write_trailer(oc);

  (* Close each codec. *)
  if Assigned(video_st) then
    close_video(oc, video_st);
  if Assigned(audio_st) then
    close_audio(oc, audio_st);

  if (fmt.flags and AVFMT_NOFILE) = 0 then
    (* Close the output file. *)
    avio_close(oc.pb);

  (* free the stream *)
  avformat_free_context(oc);

  Result := 0;
end;

begin
  try
    ExitCode := main();
  except
    on E: Exception do
      Writeln(ErrOutput, E.ClassName, ': ', E.Message);
  end;
end.
