(*
 * Copyright (C) 2011-2013 Michael Niedermayer (michaelni@gmx.at)
 *
 * This file is part of libswresample
 *
 * libswresample is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * libswresample is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with libswresample; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 *)

(**
 * @file
 * @ingroup lswr
 * libswresample public header
 *)

(*
 * FFVCL - Delphi FFmpeg VCL Components
 * http://www.DelphiFFmpeg.com
 *
 * Original file: libswresample/swresample.h
 * Ported by CodeCoolie@CNSW 2011/11/29 -> $Date:: 2014-07-20 #$
 *)

(*
FFmpeg Delphi/Pascal Headers and Examples License Agreement

A modified part of FFVCL - Delphi FFmpeg VCL Components.
Copyright (c) 2008-2014 DelphiFFmpeg.com
All rights reserved.
http://www.DelphiFFmpeg.com

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:
1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.

This source code is provided "as is" by DelphiFFmpeg.com without
warranty of any kind, either expressed or implied, including but not
limited to the implied warranties of merchantability and/or fitness
for a particular purpose.

Please also notice the License agreement of FFmpeg libraries.
*)

unit libswresample;

interface

{$I CompilerDefines.inc}

uses
  FFTypes,
  libavutil_log,
  libavutil_samplefmt;

{$I libversion.inc}

(**
 * @defgroup lswr Libswresample
 * @{
 *
 * Libswresample (lswr) is a library that handles audio resampling, sample
 * format conversion and mixing.
 *
 * Interaction with lswr is done through SwrContext, which is
 * allocated with swr_alloc() or swr_alloc_set_opts(). It is opaque, so all parameters
 * must be set with the @ref avoptions API.
 *
 * For example the following code will setup conversion from planar float sample
 * format to interleaved signed 16-bit integer, downsampling from 48kHz to
 * 44.1kHz and downmixing from 5.1 channels to stereo (using the default mixing
 * matrix):
 * @code
 * SwrContext *swr = swr_alloc();
 * av_opt_set_channel_layout(swr, "in_channel_layout",  AV_CH_LAYOUT_5POINT1, 0);
 * av_opt_set_channel_layout(swr, "out_channel_layout", AV_CH_LAYOUT_STEREO,  0);
 * av_opt_set_int(swr, "in_sample_rate",     48000,                0);
 * av_opt_set_int(swr, "out_sample_rate",    44100,                0);
 * av_opt_set_sample_fmt(swr, "in_sample_fmt",  AV_SAMPLE_FMT_FLTP, 0);
 * av_opt_set_sample_fmt(swr, "out_sample_fmt", AV_SAMPLE_FMT_S16,  0);
 * @endcode
 *
 * Once all values have been set, it must be initialized with swr_init(). If
 * you need to change the conversion parameters, you can change the parameters
 * as described above, or by using swr_alloc_set_opts(), then call swr_init()
 * again.
 *
 * The conversion itself is done by repeatedly calling swr_convert().
 * Note that the samples may get buffered in swr if you provide insufficient
 * output space or if sample rate conversion is done, which requires "future"
 * samples. Samples that do not require future input can be retrieved at any
 * time by using swr_convert() (in_count can be set to 0).
 * At the end of conversion the resampling buffer can be flushed by calling
 * swr_convert() with NULL in and 0 in_count.
 *
 * The delay between input and output, can at any time be found by using
 * swr_get_delay().
 *
 * The following code demonstrates the conversion loop assuming the parameters
 * from above and caller-defined functions get_input() and handle_output():
 * @code
 * uint8_t **input;
 * int in_samples;
 *
 * while (get_input(&input, &in_samples)) {
 *     uint8_t *output;
 *     int out_samples = av_rescale_rnd(swr_get_delay(swr, 48000) +
 *                                      in_samples, 44100, 48000, AV_ROUND_UP);
 *     av_samples_alloc(&output, NULL, 2, out_samples,
 *                      AV_SAMPLE_FMT_S16, 0);
 *     out_samples = swr_convert(swr, &output, out_samples,
 *                                      input, in_samples);
 *     handle_output(output, out_samples);
 *     av_freep(&output);
 * }
 * @endcode
 *
 * When the conversion is finished, the conversion
 * context and everything associated with it must be freed with swr_free().
 * There will be no memory leak if the data is not completely flushed before
 * swr_free().
 *)

const
{$IF LIBSWRESAMPLE_VERSION_MAJOR < 1}
  SWR_CH_MAX = 32;   ///< Maximum number of channels
{$IFEND}

  SWR_FLAG_RESAMPLE = 1; ///< Force resampling even if equal sample rate
//TODO use int resample ?
//long term TODO can we enable this dynamically?

type
  TSwrDitherType = (
    SWR_DITHER_NONE = 0,
    SWR_DITHER_RECTANGULAR,
    SWR_DITHER_TRIANGULAR,
    SWR_DITHER_TRIANGULAR_HIGHPASS,

    SWR_DITHER_NS = 64,         ///< not part of API/ABI
    SWR_DITHER_NS_LIPSHITZ,
    SWR_DITHER_NS_F_WEIGHTED,
    SWR_DITHER_NS_MODIFIED_E_WEIGHTED,
    SWR_DITHER_NS_IMPROVED_E_WEIGHTED,
    SWR_DITHER_NS_SHIBATA,
    SWR_DITHER_NS_LOW_SHIBATA,
    SWR_DITHER_NS_HIGH_SHIBATA,
    SWR_DITHER_NB               ///< not part of API/ABI
  );

(** Resampling Engines *)
  TSwrEngine = (
    SWR_ENGINE_SWR,             (**< SW Resampler *)
    SWR_ENGINE_SOXR,            (**< SoX Resampler *)
    SWR_ENGINE_NB               ///< not part of API/ABI
  );

(** Resampling Filter Types *)
  TSwrFilterType = (
    SWR_FILTER_TYPE_CUBIC,              (**< Cubic *)
    SWR_FILTER_TYPE_BLACKMAN_NUTTALL,   (**< Blackman Nuttall Windowed Sinc *)
    SWR_FILTER_TYPE_KAISER              (**< Kaiser Windowed Sinc *)
  );

  PPSwrContext = ^PSwrContext;
  PSwrContext = ^TSwrContext;
  TSwrContext = record
    // need {$ALIGN 8}
    // defined in libswresample/swresample_internal.h
  end;

(**
 * Get the AVClass for swrContext. It can be used in combination with
 * AV_OPT_SEARCH_FAKE_OBJ for examining options.
 *
 * @see av_opt_find().
 *)
function swr_get_class: PAVClass; cdecl; external SWRESAMPLE_LIBNAME name _PU + 'swr_get_class';

(**
 * Allocate SwrContext.
 *
 * If you use this function you will need to set the parameters (manually or
 * with swr_alloc_set_opts()) before calling swr_init().
 *
 * @see swr_alloc_set_opts(), swr_init(), swr_free()
 * @return NULL on error, allocated context otherwise
 *)
function swr_alloc: PSwrContext; cdecl; external SWRESAMPLE_LIBNAME name _PU + 'swr_alloc';

(**
 * Initialize context after user parameters have been set.
 *
 * @return AVERROR error code in case of failure.
 *)
function swr_init(s: PSwrContext): Integer; cdecl; external SWRESAMPLE_LIBNAME name _PU + 'swr_init';

(**
 * Check whether an swr context has been initialized or not.
 *
 * @return positive if it has been initialized, 0 if not initialized
 *)
function swr_is_initialized(s: PSwrContext): Integer; cdecl; external SWRESAMPLE_LIBNAME name _PU + 'swr_is_initialized';

(**
 * Allocate SwrContext if needed and set/reset common parameters.
 *
 * This function does not require s to be allocated with swr_alloc(). On the
 * other hand, swr_alloc() can use swr_alloc_set_opts() to set the parameters
 * on the allocated context.
 *
 * @param s               Swr context, can be NULL
 * @param out_ch_layout   output channel layout (AV_CH_LAYOUT_* )
 * @param out_sample_fmt  output sample format (AV_SAMPLE_FMT_* ).
 * @param out_sample_rate output sample rate (frequency in Hz)
 * @param in_ch_layout    input channel layout (AV_CH_LAYOUT_* )
 * @param in_sample_fmt   input sample format (AV_SAMPLE_FMT_* ).
 * @param in_sample_rate  input sample rate (frequency in Hz)
 * @param log_offset      logging level offset
 * @param log_ctx         parent logging context, can be NULL
 *
 * @see swr_init(), swr_free()
 * @return NULL on error, allocated context otherwise
 *)
function swr_alloc_set_opts(s: PSwrContext;
                                out_ch_layout: Int64; out_sample_fmt: TAVSampleFormat; out_sample_rate: Integer;
                                in_ch_layout: Int64; in_sample_fmt: TAVSampleFormat; in_sample_rate: Integer;
                                log_offset: Integer; log_ctx: Pointer): PSwrContext; cdecl; external SWRESAMPLE_LIBNAME name _PU + 'swr_alloc_set_opts';

(**
 * Free the given SwrContext and set the pointer to NULL.
 *)
procedure swr_free(s: PPSwrContext); cdecl; external SWRESAMPLE_LIBNAME name _PU + 'swr_free';

(**
 * Convert audio.
 *
 * in and in_count can be set to 0 to flush the last few samples out at the
 * end.
 *
 * If more input is provided than output space then the input will be buffered.
 * You can avoid this buffering by providing more output space than input.
 * Convertion will run directly without copying whenever possible.
 *
 * @param s         allocated Swr context, with parameters set
 * @param out       output buffers, only the first one need be set in case of packed audio
 * @param out_count amount of space available for output in samples per channel
 * @param in        input buffers, only the first one need to be set in case of packed audio
 * @param in_count  number of input samples available in one channel
 *
 * @return number of samples output per channel, negative value on error
 *)
function swr_convert(s: PSwrContext; out_: PPByte; out_count: Integer;
                              const in_: PPByte; in_count: Integer): Integer; cdecl; external SWRESAMPLE_LIBNAME name _PU + 'swr_convert';

(**
 * Convert the next timestamp from input to output
 * timestamps are in 1/(in_sample_rate * out_sample_rate) units.
 *
 * @note There are 2 slightly differently behaving modes.
 *       First is when automatic timestamp compensation is not used, (min_compensation >= FLT_MAX)
 *              in this case timestamps will be passed through with delays compensated
 *       Second is when automatic timestamp compensation is used, (min_compensation < FLT_MAX)
 *              in this case the output timestamps will match output sample numbers
 *
 * @param pts   timestamp for the next input sample, INT64_MIN if unknown
 * @return the output timestamp for the next output sample
 *)
function swr_next_pts(s: PSwrContext; pts: Int64): Int64; cdecl; external SWRESAMPLE_LIBNAME name _PU + 'swr_next_pts';

(**
 * Activate resampling compensation.
 *)
function swr_set_compensation(s: PSwrContext; sample_delta, compensation_distance: Integer): Integer; cdecl; external SWRESAMPLE_LIBNAME name _PU + 'swr_set_compensation';

(**
 * Set a customized input channel mapping.
 *
 * @param s           allocated Swr context, not yet initialized
 * @param channel_map customized input channel mapping (array of channel
 *                    indexes, -1 for a muted channel)
 * @return AVERROR error code in case of failure.
 *)
function swr_set_channel_mapping(s: PSwrContext; const channel_map: PInteger): Integer; cdecl; external SWRESAMPLE_LIBNAME name _PU + 'swr_set_channel_mapping';

(**
 * Set a customized remix matrix.
 *
 * @param s       allocated Swr context, not yet initialized
 * @param matrix  remix coefficients; matrix[i + stride * o] is
 *                the weight of input channel i in output channel o
 * @param stride  offset between lines of the matrix
 * @return  AVERROR error code in case of failure.
 *)
function swr_set_matrix(s: PSwrContext; const matrix: PDouble; stride: Integer): Integer; cdecl; external SWRESAMPLE_LIBNAME name _PU + 'swr_set_matrix';

(**
 * Drops the specified number of output samples.
 *)
function swr_drop_output(s: PSwrContext; count: Integer): Integer; cdecl; external SWRESAMPLE_LIBNAME name _PU + 'swr_drop_output';

(**
 * Injects the specified number of silence samples.
 *)
function swr_inject_silence(s: PSwrContext; count: Integer): Integer; cdecl; external SWRESAMPLE_LIBNAME name _PU + 'swr_inject_silence';

(**
 * Gets the delay the next input sample will experience relative to the next output sample.
 *
 * Swresample can buffer data if more input has been provided than available
 * output space, also converting between sample rates needs a delay.
 * This function returns the sum of all such delays.
 * The exact delay is not necessarily an integer value in either input or
 * output sample rate. Especially when downsampling by a large value, the
 * output sample rate may be a poor choice to represent the delay, similarly
 * for upsampling and the input sample rate.
 *
 * @param s     swr context
 * @param base  timebase in which the returned delay will be
 *              if its set to 1 the returned delay is in seconds
 *              if its set to 1000 the returned delay is in milli seconds
 *              if its set to the input sample rate then the returned delay is in input samples
 *              if its set to the output sample rate then the returned delay is in output samples
 *              an exact rounding free delay can be found by using LCM(in_sample_rate, out_sample_rate)
 * @returns     the delay in 1/base units.
 *)
function swr_get_delay(s: PSwrContext; base: Int64): Int64; cdecl; external SWRESAMPLE_LIBNAME name _PU + 'swr_get_delay';

(**
 * Return the LIBSWRESAMPLE_VERSION_INT constant.
 *)
function swresample_version: Cardinal; cdecl; external SWRESAMPLE_LIBNAME name _PU + 'swresample_version';

(**
 * Return the swr build-time configuration.
 *)
function swresample_configuration: PAnsiChar; cdecl; external SWRESAMPLE_LIBNAME name _PU + 'swresample_configuration';

(**
 * Return the swr license.
 *)
function swresample_license: PAnsiChar; cdecl; external SWRESAMPLE_LIBNAME name _PU + 'swresample_license';

(**
 * @}
 *)

implementation

end.
